//#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>

#include <math.h>
#include <limits.h>
#define FREE_ARG char*

static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

void normalize2(double **before, unsigned char **after, int row, int column, int level)
{ 
	int i,j;
    double max = -1000;//LONG_MIN;
	double min =  1000;//LONG_MAX;

	for ( i = 0; i < row; i++ )
		for ( j = 0; j < column; j++ ){
			if ( before[i][j] > max ) {max = before[i][j]; /*printf("%f\n",before[i][j]);*/}
			if ( before[i][j] < min ) min = before[i][j];
		}

//printf("max = %f min = %f\n",max,min);

	for ( i = 0; i < row; i++ )
		for ( j = 0; j < column; j++ ){
			after[i][j] = (unsigned char) (( (before[i][j] - min) * level) / ( max - min));
            //if (after[i][j] == 255) printf("%d\n",after[i][j]);
		}

}

void nrerror(char *error_text)
// Numerical Recipes standard error handler
{
	fprintf(stderr, "%s\n", error_text);
	fprintf(stderr, "...now exiting to system...\n");
	exit(0);
}

double pythag(double a, double b)
{
	double	absa, absb;

	absa = fabs(a);
	absb = fabs(b);

	if(absa>absb)	return absa*sqrt(1.0+SQR(absb/absa));
	else	return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));

}

float *vector(long nl, long nh)
// allocate a float vector with subscript range v[nl...nh]
{
	float *v;

	v=(float *)malloc((size_t) ((nh-nl+1)*sizeof(float)));
	if(!v)	nrerror("allocation failure in vector()");
	return v-nl;
}

double *dvector(long start, long end)
{
	double *v;

	v = (double *)malloc((end-start+1)*sizeof(double));

	return v - start;
}

unsigned char *ucvector(int start, int end)
{
	unsigned char *v;

	v = (unsigned char *)malloc((end-start+1)*sizeof(unsigned char));

	return v - start;
}

unsigned short *usvector(int start, int end)
{
	unsigned short *v;

	v = (unsigned short *)malloc((end-start+1)*sizeof(unsigned short));

	return v - start;
}

int *ivector(int start, int end)
{
	int *v;

	v = (int *)malloc((end-start+1)*sizeof(int));

	return v - start;
}

double **dpvector(int start, int end)
{
	double **v;

	v = (double **)malloc((end-start+1)*sizeof(double*));

	return v - start;
}

unsigned char **ucpvector(int start, int end)
{
	unsigned char **v;

	v = (unsigned char **)malloc((end-start+1)*sizeof(unsigned char*));

	return v - start;
}

int **ipvector(int start, int end)
{
	int **v;

	v = (int **)malloc((end-start+1)*sizeof(int*));

	return v - start;
}

float **matrix(long nrl, long nrh, long ncl, long nch)
// allocate a float matrix with subscript range m[nrl...nrh][ncl...nch]
{
	long	i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
	float	**m;

	m=(float **)malloc((size_t) ((nrow)*sizeof(float*)));
	if(!m)	nrerror("allocation failure 1 in matrix()");
	m -= nrl;
	
	m[nrl] = (float *)malloc((size_t)((nrow*ncol)*sizeof(float)));
	if(!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] -= ncl;
	
	for(i=nrl+1; i<=nrh; i++)
		m[i] = m[i-1]+ncol;
		
	return m;
}

double **dmatrix(long r_start, long r_end, long c_start, long c_end)
{
	int i,row = r_end - r_start +1, column = c_end - c_start +1;
	double **m;
    //double **copy;
    //int j;
    //int x = 1;

	m = (double **)malloc(row*sizeof(double *));  
	//copy = m;
	m -= r_start;

	m[r_start] = (double *)malloc(row*column*sizeof(double));
/*    for ( i = 0; i <= row-1; i++ )
		for ( j = 0; j <= column-1; j++ ){
		    *(copy[0]+i*column+j) = x;
            x++;
		}
*/
	m[r_start] -= c_start;

/*
    for ( i = 4; i <= 19; i++ )
		printf("%f\n",*(copy[0]+i));
*/

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

    //if ( copy[0] == m[r_start] ) printf("ok\n");
    //if ( copy[0][4] == m[r_start][c_start] ) printf("Ouf!\n");

	return m;
}

double ***dcube(int r_start, int r_end, int c_start, int c_end, int cu_start, int cu_end)
{
	int i,j;
	int row = r_end - r_start +1, column = c_end - c_start +1;
	int width = cu_end - cu_start + 1;
	double ***m;
    //double ***copy;
    //int x = 1;

	m = (double ***)malloc(row*sizeof(double **));
	//copy = m;
	
	m -= r_start;

	m[r_start] = (double **)malloc(row*column*sizeof(double*));
	m[r_start] -= c_start;

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

	m[r_start][c_start] = (double *)malloc(row*column*width*sizeof(double));

/*
    for ( i = 0; i <= row-1; i++ )
	    for ( j = 0; j <= column-1; j++ )
			for ( k = 0; k <= width-1; k++ ){
		    *(copy[0][1]+i*column*width+j*width+k) = x;
			//printf("%d\n",i*column*width+j*width+k);
            x++;
		}
*/
	//for ( i = 0; i <= row*column*width-1; i++ )
		//printf("%f\n",*(copy[0][1]+i));

	m[r_start][c_start] -= cu_start;

	for ( i = r_start; i <= r_end; i++ )
		for ( j = c_start; j <= c_end; j++ )
			if ( i != r_start || j != c_start ){//printf("i = %d j = %d\n",i,j);
			   if ( j != c_start )
                   m[i][j] = m[i][j-1] + width;
               if ( j == c_start )
				   m[i][j] = m[i-1][j] + column*width;
			}

	return m;
}

unsigned char ***uccube(int r_start, int r_end, int c_start, int c_end, int cu_start, int cu_end)
{
	int i,j;
	int row = r_end - r_start +1, column = c_end - c_start +1;
	int width = cu_end - cu_start + 1;
	unsigned char ***m;

	m = (unsigned char ***)malloc(row*sizeof(unsigned char **));
	
	m -= r_start;

	m[r_start] = (unsigned char **)malloc(row*column*sizeof(unsigned char*));
	m[r_start] -= c_start;

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

	m[r_start][c_start] = (unsigned char *)malloc(row*column*width*sizeof(unsigned char));

	m[r_start][c_start] -= cu_start;

	for ( i = r_start; i <= r_end; i++ )
		for ( j = c_start; j <= c_end; j++ )
			if ( i != r_start || j != c_start ){//printf("i = %d j = %d\n",i,j);
			   if ( j != c_start )
                   m[i][j] = m[i][j-1] + width;
               if ( j == c_start )
				   m[i][j] = m[i-1][j] + column*width;
			}

	return m;
}

double ****d4Darray(int r_start, int r_end, int c_start, int c_end, int cu_start, int cu_end, int D4_start, int D4_end )
{
	int i,j,k;
	int row = r_end - r_start +1, column = c_end - c_start +1;
	int width = cu_end - cu_start + 1;
    int time  = D4_end - D4_start + 1;
	double ****m;

	m = (double ****)malloc(row*sizeof(double ***));
    m -= r_start;

	m[r_start] = (double ***)malloc(row*column*sizeof(double**));
	m[r_start] -= c_start;

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

	m[r_start][c_start] = (double **)malloc(row*column*width*sizeof(double*));
	m[r_start][c_start] -= cu_start;

	for ( i = r_start; i <= r_end; i++ )
		for ( j = c_start; j <= c_end; j++ )
			if ( i != r_start || j != c_start ){
			   if ( j != c_start )
                   m[i][j] = m[i][j-1] + width;
               if ( j == c_start )
				   m[i][j] = m[i-1][j] + column*width;
			}

    m[r_start][c_start][cu_start] = (double *)malloc(row*column*width*time*sizeof(double));
    m[r_start][c_start][cu_start] -= D4_start;

	for ( i = r_start; i <= r_end; i++ )
		for ( j = c_start; j <= c_end; j++ )
			for ( k = cu_start; k <= cu_end; k++ )
				if ( i != r_start || j != c_start || k != cu_start ){
				   if ( k != cu_start ){
				      m[i][j][k] = m[i][j][k-1] + time;}
				   if ( k == cu_start && j != c_start ){
				      m[i][j][k] = m[i][j-1][k] + width*time;}
				   if ( k == cu_start && j == c_start ){
				      m[i][j][k] = m[i-1][j][k] + column*width*time;}
				}

	return m;
}

unsigned char **ucmatrix(int r_start, int r_end, int c_start, int c_end)
{
	int i,row = r_end - r_start +1, column = c_end - c_start +1;
	unsigned char **m;

	m = (unsigned char **)malloc(row*sizeof(unsigned char *));  

	m -= r_start;

	m[r_start] = (unsigned char *)malloc(row*column*sizeof(unsigned char));

	m[r_start] -= c_start;

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

	return m;
}

int **imatrix(int r_start, int r_end, int c_start, int c_end)
{
	int i,row = r_end - r_start +1, column = c_end - c_start +1;
	int **m;

	m = (int **)malloc(row*sizeof(int *));  

	m -= r_start;

	m[r_start] = (int *)malloc(row*column*sizeof(int));

	m[r_start] -= c_start;

	for ( i = r_start + 1; i <= r_end; i++ )
		m[i] = m[i-1] + column;

	return m;
}

void free_vector(float *v, long nl, long nh)
// free a float vector allocated with vector()
{
	free((FREE_ARG) (v+nl));
}

void free_dvector(double *v, long nl, long nh)
// free a double vector allocated with dvector()
{
	free((FREE_ARG) (v+nl));
}

void free_ivector(int *v, long nl, long nh)
// free a double vector allocated with ivector()
{
	free((FREE_ARG) (v+nl));
}

void free_ucvector(unsigned char *v, long nl, long nh)
{
	free((FREE_ARG) (v+nl));
}

void free_usvector(unsigned short *v, long nl, long nh)
{
	free((FREE_ARG) (v+nl));
}

void free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
// free a float matrix allocated by matrix()
{
	free((FREE_ARG) (m[nrl]+ncl));
	free((FREE_ARG) (m+nrl));
}

void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
// free a double matrix allocated by dmatrix()
{
	free((FREE_ARG) (m[nrl]+ncl));
	free((FREE_ARG) (m+nrl));
}

void free_ucmatrix(unsigned char **m, long nrl, long nrh, long ncl, long nch)
// free a double matrix allocated by dmatrix()
{
	free((FREE_ARG) (m[nrl]+ncl));
	free((FREE_ARG) (m+nrl));
}

void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
// free a double matrix allocated by dmatrix()
{
	free((FREE_ARG) (m[nrl]+ncl));
	free((FREE_ARG) (m+nrl));
}
