//#include "stdafx.h"


#include <stdio.h>
#include <math.h>
#include "memory_allocate.h"
#include "svd_v2.h"
#include "Table.h"

void svdcmp(double **a, int m, int n, double *w, double **v)
{
	int		flag, i, its, j, jj, k, l, nm;
	double	anorm, c, f, g, h, s, scale, x, y, z, *rv1;
	double  **ev_temp,**ev_temp2;

	ev_temp  = dmatrix(1,m,1,n);
	ev_temp2 = dmatrix(1,n,1,n);

	rv1 = dvector(1, n);
	g = scale = anorm = 0.0;							// Householder reduction to bidiagonal form.
	for(i=1; i<=n; i++){
		l=i+1;
		rv1[i]=scale*g;
		g = s = scale = 0.0;
		if(i<=m){
			for(k=i; k<=m; k++)	scale += fabs(a[k][i]);
			if (scale){
				for(k=i; k<=m; k++){
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f = a[i][i];
				g = -SIGN(sqrt(s), f);
				h = f*g-s;
				a[i][i] = f-g;
				for(j=l; j<=n; j++){
					for(s=0.0, k=i; k<=m; k++) s += a[k][i]*a[k][j];
					f = s/h;
					for(k=i; k<=m; k++) a[k][j] += f*a[k][i];
				}
				for(k=i; k<=m; k++) a[k][i] *= scale;
			}
		}
		w[i] = scale*g;
		g = s = scale = 0.0;
		if(i<=m && i!=n){
			for(k=l; k<=n; k++)	scale += fabs(a[i][k]);
			if(scale){
				for(k=l; k<=n; k++){
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f = a[i][l];
				g = -SIGN(sqrt(s), f);
				h = f*g-s;
				a[i][l] = f-g;
				for(k=l; k<=n; k++) rv1[k] = a[i][k]/h;
				for(j=l; j<=m; j++){
					for(s=0.0, k=l; k<=n; k++) s += a[j][k]*a[i][k];
					for(k=l; k<=n; k++)	a[j][k] += s*rv1[k];
				}
				for(k=l; k<=n; k++) a[i][k] *= scale;
			}
		}
		anorm = FMAX(anorm, (fabs(w[i])+fabs(rv1[i])));
	}
	for(i=n; i>=1; i--){								// Accumulation of right-hand transformations.
		if(i<n){
			if(g){
				for(j=l; j<=n; j++)						// Double division to avoid possible underflow.
					v[j][i] = (a[i][j]/a[i][l])/g;
				for(j=l; j<=n; j++){
					for(s=0.0, k=l; k<=n; k++) s += a[i][k]*v[k][j];
					for(k=l; k<=n; k++) v[k][j] += s*v[k][i];
				}
			}
			for(j=l; j<=n; j++) v[i][j] = v[j][i] = 0.0;
		}
		v[i][i] = 1.0;
		g = rv1[i];
		l = i;
	}
	for(i=IMIN(m, n); i>=1; i--){						// Accumulation of left-hand transformations.
		l = i+1;
		g = w[i];
		for(j=l; j<=n; j++) a[i][j] = 0.0;
		if(g){
			g = 1.0/g;
			for(j=l; j<=n; j++){
				for(s=0.0, k=l; k<=m; k++) s += a[k][i]*a[k][j];
				f = (s/a[i][i])*g;
				for(k=i; k<=m; k++) a[k][j] += f*a[k][i];
			}
			for(j=i; j<=m; j++) a[j][i] *= g;
		} else for(j=i; j<=m; j++) a[j][i] = 0.0;
		++a[i][i];
	}
	for(k=n; k>=1; k--){								// Diagonalization of the bidiagonal form: Loop over
		for(its=1; its<=50; its++){						//		singular values, and over allowed iterations.
			flag = 1;
			for(l=k; l>=1; l--){						// Test for splitting.
				nm = l-1;								// Note that rv[1] is always zero.
				if((fabs(rv1[l])+anorm) == anorm){
					flag = 0;
					break;
				}
				if((fabs(w[nm])+anorm) == anorm) break;
			}
			if(flag){
				c = 0.0;								// Cancellation of rv1[l], if l>1.
				s = 1.0;
				for(i=l; i<=k; i++){
					f = s*rv1[i];
					rv1[i] = c*rv1[i];
					if((fabs(f)+anorm) == anorm) break;
					g = w[i];
					h = pythag(f, g);
					w[i] = h;
					h = 1.0/h;
					c = g*h;
					s = -f*h;
					for(j=1; j<=m; j++){
						y = a[j][nm];
						z = a[j][i];
						a[j][nm] = y*c+z*s;
						a[j][i] = z*c-y*s;
					}
				}
			}
			z = w[k];
			if(l == k){									// Convergence
				if(z<0.0){								// Singular value if made nonnegative
					w[k] = -z;
					for(j=1; j<=n; j++)	v[j][k] = -v[j][k];
				}
				break;
			}
			if(its == 50)	nrerror("no convergence in 50 dsvdcmp iterations");
			x = w[l];										// Shift from bottom 2-by-2 minor
			nm = k-1;
			y = w[nm];
			g = rv1[nm];
			h = rv1[k];
			f = ((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g = pythag(f, 1.0);
			f = ((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c = s = 1.0;
			for(j=l; j<=nm; j++){
				i = j+1;
				g = rv1[i];
				y = w[i];
				h = s*g;
				g = c*g;
				z = pythag(f, h);
				rv1[j] = z;
				c = f/z;
				s = h/z;
				f = x*c+g*s;
				g = g*c-x*s;
				h = y*s;
				y *= c;
				for(jj=1; jj<=n; jj++){
					x = v[jj][j];
					z = v[jj][i];
					v[jj][j] = x*c+z*s;
					v[jj][i] = z*c-x*s;
				}
				z = pythag(f, h);
				w[j] = z;								// Rotation can be arbitary if z=0
				if(z){
					z = 1.0/z;
					c = f*z;
					s = h*z;
				}
				f = c*g+s*y;
				x = c*y-s*g;
				for(jj=1; jj<=m; jj++){
					y = a[jj][j];
					z = a[jj][i];
					a[jj][j] = y*c+z*s;
					a[jj][i] = z*c-y*s;
				}
			}
			rv1[l] = 0.0;
			rv1[k] = f;
			w[k] = x;
		}
	}

    for ( i = 1; i <= m; i++ )
		for ( j = 1; j <= n; j++ )
			ev_temp[i][j] = a[i][j];

	for ( i = 1; i <= n; i++ )
		for ( j = 1; j <= n; j++ )
			ev_temp2[i][j] = v[i][j];
  
    int *nc;

	nc = ivector(1, n);
	for (i=1; i<=n; i++)
		nc[i] = i;

    double wrk;
	int itemp;

	//------- Sort eigenvalues 

	for(j=1; j<=n-1; j++){
		for(k=j; k<=n; k++){
			if(w[j]<w[k]){
				wrk=w[j];
				w[j]=w[k];
				w[k]=wrk;
				itemp=nc[j];
				nc[j]=nc[k];
				nc[k]=itemp;
			}
		}
	}

	//-------- Sort eigen vector 

	for(i=1;i<=n;i++){
		k=nc[i];
		for(j=1;j<=m;j++){
			a[j][i] = ev_temp[j][k];
		}
	}

	for(i=1;i<=n;i++){
		k=nc[i];
		for(j=1;j<=n;j++){
			v[j][i] = ev_temp2[j][k];
		}
	}

    free_ivector(nc,1,n);
	free_dvector(rv1, 1, n);
	free_dmatrix(ev_temp,1,m,1,n);
	free_dmatrix(ev_temp2,1,n,1,n);
}

void svdcmp_zeromat(double **a, int m, int n, double *w, double **v)//
{
	int		i,j,d;
	double  **right,**left,*ww;

	if(m<n)d=n;
	else d=m;

	right=dmatrix(1,m,1,n);
	left=dmatrix(1,n,1,n);
	ww=dvector(1,d);
 
	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			right[i+1][j+1]=a[i][j];
	}}
	
    svdcmp(right,m,n,ww,left);

	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			a[i][j]=right[i+1][j+1];
	}}

	for(i=0;i<n;i++){
		for(j=0;j<n;j++){
			v[i][j]=left[i+1][j+1];
	}}

	for(i=0;i<n;i++){
		w[i]=ww[i+1];
	}

	free_dvector(ww,1,m);
	free_dmatrix(right,1,m,1,n);
	free_dmatrix(left,1,n,1,n);

}

void svdinv_zeromat(double **a, int m, int n, double **inv_a)//m>n̎
{
	int		i,j,k,d;
	double  **v,*w,**a_tmp;

	if(m<n)d=n;
	else d=m;

	a_tmp=dmatrix(0,m,0,n);
	w=dvector(0,d);
	v=dmatrix(0,n,0,n);

	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			a_tmp[i][j]=a[i][j];
	}}

	svdcmp_zeromat(a_tmp,m,n,w,v);

	for(i=0;i<n;i++){
		for(j=0;j<m;j++){
			inv_a[i][j]=0;
		}
	}

	for(k=0;k<d;k++){
		if(w[k]>1E-10){
			for(i=0;i<n;i++){
				for(j=0;j<m;j++){
					inv_a[i][j]+=v[i][k]*a_tmp[j][k]/w[k];
				}
			}
		}
	}

	free_dmatrix(a_tmp,0,m,0,n);
	free_dmatrix(v,0,n,0,n);
	free_dvector(w,0,d);

}

void svdinv_zeromat1D(double *mat,double *inv,int Dim,int Num)
{//mat(Dim*Num) -> inv(Dim*Num)
	int i,j;
	double **mat2D,**inv2D;
	
	mat2D=dmatrix(0,Dim,0,Num);
	inv2D=dmatrix(0,Num,0,Dim);
	
	for(i=0;i<Dim;i++){
		for(j=0;j<Num;j++){
			mat2D[i][j]=mat[i*Num+j];
		} 
	}
	svdinv_zeromat(mat2D,Dim,Num,inv2D);
	
	for(i=0;i<Num;i++){
		for(j=0;j<Dim;j++){
			inv[i*Dim+j]=inv2D[i][j];//
		} 
	}
	
	
	free_dmatrix(mat2D,0,Dim,0,Num);
	free_dmatrix(inv2D,0,Num,0,Dim);
	
}

double* GetSVDInverse(char *fn,int *Num,int *BAND)
{
	double *mat;
	double *inv;


	mat=DataGet(fn,Num,BAND);
	inv=dvector(0,(*Num)*(*BAND));
	svdinv_zeromat1D(mat,inv,*Num,*BAND);

	free_dvector(mat,0,(*Num)*(*BAND));

	return inv;

}

void Fwrite_SVDInverse(char *fn)
{
	
	char refn[200];
	int Num,BAND;
	double *inv;

	ReName(fn,refn,"_inv.sig");

	inv=GetSVDInverse(fn,&Num,&BAND);
	Fwrite_TABLE(refn,BAND,Num,inv);	

	printf("SVD Inverse matrix is in %s File\n",refn);
	printf("The matrix is (%d x %d ) \n",BAND,Num);

	free_dvector(inv,0,BAND*Num);

//	getchar();
	
}

void weightsvd(void)
{
	char *fn_dye="R(opt)R(overE)H_E_tra16.txt";//mat[BAND*Num]
	char *fn_xyz="XYZave_tra16.txt";//W[BAND]


	int i,j,k;
	int Num,BAND;
	int wNum;

	double *mat;
	double *cmat;
	double *temp,*itemp;
	double *W;
	double *inv;
	char refn[200];

	mat=DataGet(fn_dye,&BAND,&Num); //mat[BAND*Num]
	cmat=dvector(0,BAND*Num);//cmat[BAND*Num]
	temp=dvector(0,Num*Num);
	itemp=dvector(0,Num*Num);
	W=DataGet(fn_xyz,&BAND,&wNum);//W[BAND]
	inv=dvector(0,BAND*Num);//inv[Num*BAND]

	for(i=0;i<BAND;i++){
		for(j=0;j<Num;j++){
			cmat[i*Num+j]=mat[i*Num+j]*W[i];
		}
	}

	for(i=0;i<Num;i++){
		for(j=0;j<Num;j++){
			temp[i*Num+j]=0;
			for(k=0;k<BAND;k++){
				temp[i*Num+j]+=cmat[k*Num+i]*mat[k*Num+j];
			}
		}
	}

	svdinv_zeromat1D(temp,itemp,Num,Num);
	Fwrite_TABLE("test.xls",Num,Num,itemp);	

	for(i=0;i<Num;i++){
		for(j=0;j<BAND;j++){
			inv[i*BAND+j]=0;
			for(k=0;k<Num;k++){
				inv[i*BAND+j]+=itemp[i*Num+k]*cmat[j*Num+k];
			}
		}
	}


	ReName(fn_dye,refn,"_inv.sig");

	Fwrite_TABLE(refn,Num,BAND,inv);	

	printf("SVD Inverse matrix is in %s File\n",refn);
	printf("The matrix is (%d x %d ) \n",BAND,Num);


	free_dvector(mat,0,(Num)*(BAND));


}

void svdcmp_zeromat1D(double *u, int m, int n, double *w,double *v)
{
	int		i,j;
	double  **v2,**a_tmp;

	a_tmp=dmatrix(0,m,0,n);
	v2=dmatrix(0,n,0,n);

	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			//a_tmp[i][j]=a[i][j];
			a_tmp[i][j]=u[i*n+j];
		}
	}

	svdcmp_zeromat(a_tmp,m,n,w,v2);

	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			u[i*n+j]=a_tmp[i][j];
		}
	}

	for(i=0;i<n;i++){
		for(j=0;j<n;j++){
			v[i*n+j]=v2[i][j];
		}
	}

	free_dmatrix(a_tmp,0,m,0,n);
	free_dmatrix(v2,0,n,0,n);
	//free_dvector(w,0,d);

}

void Fwrite_SVD(char *fn){
	
	int Num;
	int Dim;
	int d;
	double *data;//f[^i[
	double *w,*v;

	char fn_u[200];
	char fn_w[200];
	char fn_v[200];

	data=DataGet(fn,&Num,&Dim);

	if(Num<Dim)d=Dim;
	else d=Num;
	
	w=dvector(0,d);
	v=dvector(0,Dim*Dim);

	svdcmp_zeromat1D(data,Num,Dim,w,v);

	ReName(fn,fn_u,"_umat.xls");
	ReName(fn,fn_w,"_wmat.xls");
	ReName(fn,fn_v,"_vmat.xls");

	Fwrite_TABLE(fn_w,Dim,1,w);
	Fwrite_TABLE(fn_u,Num,Dim,data);
	Fwrite_TABLE(fn_v,Dim,Dim,v);

	free_dvector(data,0,Num*Dim);
	free_dvector(w,0,d);
	free_dvector(v,0,Dim*Dim);

}
double* GetSVDInverse(double *mat,int Num,int BAND)
{
	double *inv;
	inv=dvector(0,(Num)*(BAND));
	svdinv_zeromat1D(mat,inv,Num,BAND);

	return inv;

}
